"use strict";
var chai_1 = require('chai');
var pubsub_1 = require('../pubsub');
var graphql_1 = require('graphql');
describe('FilteredPubSub', function () {
    it('can subscribe and is called when events happen', function (done) {
        var ps = new pubsub_1.FilteredPubSub();
        ps.subscribe('a', function () { return true; }, function (payload) {
            chai_1.expect(payload).to.equals('test');
            done();
        });
        ps.publish('a', 'test');
    });
    it('can filter events that get sent to subscribers', function (done) {
        var ps = new pubsub_1.FilteredPubSub();
        ps.subscribe('a', function (payload) { return payload !== 'bad'; }, function (payload) {
            chai_1.expect(payload).to.equals('good');
            done();
        });
        ps.publish('a', 'bad');
        ps.publish('a', 'good');
    });
    it('can unsubscribe', function (done) {
        var ps = new pubsub_1.FilteredPubSub();
        var subId = ps.subscribe('a', function () { return true; }, function (payload) {
            chai_1.assert(false);
        });
        ps.unsubscribe(subId);
        ps.publish('a', 'test');
        done();
    });
});
var schema = new graphql_1.GraphQLSchema({
    query: new graphql_1.GraphQLObjectType({
        name: 'Query',
        fields: {
            testString: {
                type: graphql_1.GraphQLString,
                resolve: function (_, args) {
                    return 'works';
                },
            },
        },
    }),
    subscription: new graphql_1.GraphQLObjectType({
        name: 'Subscription',
        fields: {
            testSubscription: {
                type: graphql_1.GraphQLString,
                resolve: function (root) {
                    return root;
                },
            },
        },
    }),
});
describe('SubscriptionManager', function () {
    var subManager = new pubsub_1.SubscriptionManager({ schema: schema });
    it('throws an error if query is not valid', function () {
        var query = 'query a{ testInt }';
        var callback = function () { return null; };
        return chai_1.expect(function () { return subManager.subscribe({ query: query, operationName: 'a', callback: callback }); }).to.throw('Error: Subscription query has validation errors');
    });
    it('rejects subscriptions with more than one root field', function () {
        var query = 'subscription X{ a: testSubscription, b: testSubscription }';
        var callback = function () { return null; };
        return chai_1.expect(function () { return subManager.subscribe({ query: query, operationName: 'X', callback: callback }); }).to.throw('Error: Subscription query has validation errors');
    });
    it('requires operationName to be provided', function () {
        var query = 'subscription { testSubscription }';
        var callback = function () { return null; };
        return chai_1.expect(function () { return subManager.subscribe({ query: query, operationName: undefined, callback: callback }); }).to.throw('Must provide operationName');
    });
    it('can subscribe with a valid query and gets a subId back', function () {
        var query = 'subscription X{ testSubscription }';
        var callback = function () { return null; };
        var subId = subManager.subscribe({ query: query, operationName: 'X', callback: callback });
        chai_1.expect(subId).to.be.a('number');
        subManager.unsubscribe(subId);
    });
    it('can subscribe with a valid query and get the root value', function (done) {
        var query = 'subscription X{ testSubscription }';
        var callback = function (err, payload) {
            try {
                chai_1.expect(payload.data.testSubscription).to.equals('good');
            }
            catch (e) {
                done(e);
                return;
            }
            done();
        };
        var subId = subManager.subscribe({ query: query, operationName: 'X', callback: callback });
        subManager.publish('X', 'good');
        subManager.unsubscribe(subId);
    });
    it('can unsubscribe', function (done) {
        var query = 'subscription X{ testSubscription }';
        var callback = function (err, payload) {
            try {
                chai_1.assert(false);
            }
            catch (e) {
                done(e);
                return;
            }
            done();
        };
        var subId = subManager.subscribe({ query: query, operationName: 'X', callback: callback });
        subManager.unsubscribe(subId);
        subManager.publish('X', 'bad');
        setTimeout(done, 5);
    });
    it('calls the error callback if there is an execution error', function (done) {
        var query = "subscription X($uga: Boolean!){\n      testSubscription  @skip(if: $uga)\n    }";
        var callback = function (err, payload) {
            try {
                chai_1.expect(payload).to.be.undefined;
                chai_1.expect(err.message).to.equals('Variable "$uga" of required type "Boolean!" was not provided.');
            }
            catch (e) {
                done(e);
                return;
            }
            done();
        };
        var subId = subManager.subscribe({ query: query, operationName: 'X', callback: callback });
        subManager.publish('X', 'good');
        subManager.unsubscribe(subId);
    });
});
//# sourceMappingURL=tests.js.map